import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { Extensions } from 'vscode/vscode/vs/workbench/common/contributions';
import { Registry } from 'vscode/vscode/vs/platform/registry/common/platform';
import { UserDataSyncWorkbenchContribution } from './userDataSync.js';
import { IUserDataAutoSyncService } from 'vscode/vscode/vs/platform/userDataSync/common/userDataSync';
import { INotificationService, Severity } from 'vscode/vscode/vs/platform/notification/common/notification';
import { Disposable } from 'vscode/vscode/vs/base/common/lifecycle';
import { localizeWithPath } from 'vscode/vscode/vs/nls';
import { isWeb } from 'vscode/vscode/vs/base/common/platform';
import { UserDataSyncTrigger } from './userDataSyncTrigger.js';
import { Action } from 'vscode/vscode/vs/base/common/actions';
import { IProductService } from 'vscode/vscode/vs/platform/product/common/productService';
import { ICommandService } from 'vscode/vscode/vs/platform/commands/common/commands';
import { IHostService } from 'vscode/vscode/vs/workbench/services/host/browser/host';
import { SHOW_SYNC_LOG_COMMAND_ID } from 'vscode/vscode/vs/workbench/services/userDataSync/common/userDataSync';

let UserDataSyncReportIssueContribution = class UserDataSyncReportIssueContribution extends Disposable {
    constructor(userDataAutoSyncService, notificationService, productService, commandService, hostService) {
        super();
        this.notificationService = notificationService;
        this.productService = productService;
        this.commandService = commandService;
        this.hostService = hostService;
        this._register(userDataAutoSyncService.onError(error => this.onAutoSyncError(error)));
    }
    onAutoSyncError(error) {
        switch (error.code) {
            case "LocalTooManyRequests" : {
                const message = isWeb ? ( localizeWithPath(
                    'vs/workbench/contrib/userDataSync/browser/userDataSync.contribution',
                    { key: 'local too many requests - reload', comment: ['Settings Sync is the name of the feature'] },
                    "Settings sync is suspended temporarily because the current device is making too many requests. Please reload {0} to resume.",
                    this.productService.nameLong
                ))
                    : ( localizeWithPath(
                    'vs/workbench/contrib/userDataSync/browser/userDataSync.contribution',
                    { key: 'local too many requests - restart', comment: ['Settings Sync is the name of the feature'] },
                    "Settings sync is suspended temporarily because the current device is making too many requests. Please restart {0} to resume.",
                    this.productService.nameLong
                ));
                this.notificationService.notify({
                    severity: Severity.Error,
                    message,
                    actions: {
                        primary: [
                            ( new Action('Show Sync Logs', ( localizeWithPath(
                                'vs/workbench/contrib/userDataSync/browser/userDataSync.contribution',
                                'show sync logs',
                                "Show Log"
                            )), undefined, true, () => this.commandService.executeCommand(SHOW_SYNC_LOG_COMMAND_ID))),
                            ( new Action('Restart', isWeb ? ( localizeWithPath(
                                'vs/workbench/contrib/userDataSync/browser/userDataSync.contribution',
                                'reload',
                                "Reload"
                            )) : ( localizeWithPath(
                                'vs/workbench/contrib/userDataSync/browser/userDataSync.contribution',
                                'restart',
                                "Restart"
                            )), undefined, true, () => this.hostService.restart()))
                        ]
                    }
                });
                return;
            }
            case "RemoteTooManyRequests" : {
                const operationId = error.operationId ? ( localizeWithPath(
                    'vs/workbench/contrib/userDataSync/browser/userDataSync.contribution',
                    'operationId',
                    "Operation Id: {0}",
                    error.operationId
                )) : undefined;
                const message = ( localizeWithPath(
                    'vs/workbench/contrib/userDataSync/browser/userDataSync.contribution',
                    { key: 'server too many requests', comment: ['Settings Sync is the name of the feature'] },
                    "Settings sync is disabled because the current device is making too many requests. Please wait for 10 minutes and turn on sync."
                ));
                this.notificationService.notify({
                    severity: Severity.Error,
                    message: operationId ? `${message} ${operationId}` : message,
                    source: error.operationId ? ( localizeWithPath(
                        'vs/workbench/contrib/userDataSync/browser/userDataSync.contribution',
                        'settings sync',
                        "Settings Sync. Operation Id: {0}",
                        error.operationId
                    )) : undefined,
                    actions: {
                        primary: [
                            ( new Action('Show Sync Logs', ( localizeWithPath(
                                'vs/workbench/contrib/userDataSync/browser/userDataSync.contribution',
                                'show sync logs',
                                "Show Log"
                            )), undefined, true, () => this.commandService.executeCommand(SHOW_SYNC_LOG_COMMAND_ID))),
                        ]
                    }
                });
                return;
            }
        }
    }
};
UserDataSyncReportIssueContribution = ( __decorate([
    ( __param(0, IUserDataAutoSyncService)),
    ( __param(1, INotificationService)),
    ( __param(2, IProductService)),
    ( __param(3, ICommandService)),
    ( __param(4, IHostService))
], UserDataSyncReportIssueContribution));
const workbenchRegistry = ( Registry.as(Extensions.Workbench));
workbenchRegistry.registerWorkbenchContribution(UserDataSyncWorkbenchContribution, 3 );
workbenchRegistry.registerWorkbenchContribution(UserDataSyncTrigger, 4 );
workbenchRegistry.registerWorkbenchContribution(UserDataSyncReportIssueContribution, 4 );
