import { __decorate, __param } from 'vscode/external/tslib/tslib.es6.js';
import { Event } from 'vscode/vscode/vs/base/common/event';
import { Disposable } from 'vscode/vscode/vs/base/common/lifecycle';
import { isWeb } from 'vscode/vscode/vs/base/common/platform';
import { isEqual } from 'vscode/vscode/vs/base/common/resources';
import { IUserDataProfilesService } from 'vscode/vscode/vs/platform/userDataProfile/common/userDataProfile';
import { IUserDataAutoSyncService } from 'vscode/vscode/vs/platform/userDataSync/common/userDataSync';
import { IViewsService } from 'vscode/vscode/vs/workbench/services/views/common/viewsService';
import { VIEWLET_ID } from 'vscode/vscode/vs/workbench/contrib/extensions/common/extensions';
import { IEditorService } from 'vscode/vscode/vs/workbench/services/editor/common/editorService';
import { IHostService } from 'vscode/vscode/vs/workbench/services/host/browser/host';
import { KeybindingsEditorInput } from 'vscode/vscode/vs/workbench/services/preferences/browser/keybindingsEditorInput';
import { SettingsEditor2Input } from 'vscode/vscode/vs/workbench/services/preferences/common/preferencesEditorInput';

let UserDataSyncTrigger = class UserDataSyncTrigger extends Disposable {
    constructor(editorService, userDataProfilesService, viewsService, userDataAutoSyncService, hostService) {
        super();
        this.userDataProfilesService = userDataProfilesService;
        const event = Event.filter(Event.any(( Event.map(
            editorService.onDidActiveEditorChange,
            () => this.getUserDataEditorInputSource(editorService.activeEditor)
        )), ( Event.map(
            Event.filter(viewsService.onDidChangeViewContainerVisibility, e => e.id === VIEWLET_ID && e.visible),
            e => e.id
        ))), source => source !== undefined);
        if (isWeb) {
            this._register(Event.debounce(Event.any(( Event.map(hostService.onDidChangeFocus, () => 'windowFocus')), ( Event.map(event, source => source))), (last, source) => last ? [...last, source] : [source], 1000)(sources => userDataAutoSyncService.triggerSync(sources, true, false)));
        }
        else {
            this._register(event(source => userDataAutoSyncService.triggerSync([source], true, false)));
        }
    }
    getUserDataEditorInputSource(editorInput) {
        if (!editorInput) {
            return undefined;
        }
        if (editorInput instanceof SettingsEditor2Input) {
            return 'settingsEditor';
        }
        if (editorInput instanceof KeybindingsEditorInput) {
            return 'keybindingsEditor';
        }
        const resource = editorInput.resource;
        if (isEqual(resource, this.userDataProfilesService.defaultProfile.settingsResource)) {
            return 'settingsEditor';
        }
        if (isEqual(resource, this.userDataProfilesService.defaultProfile.keybindingsResource)) {
            return 'keybindingsEditor';
        }
        return undefined;
    }
};
UserDataSyncTrigger = ( __decorate([
    ( __param(0, IEditorService)),
    ( __param(1, IUserDataProfilesService)),
    ( __param(2, IViewsService)),
    ( __param(3, IUserDataAutoSyncService)),
    ( __param(4, IHostService))
], UserDataSyncTrigger));

export { UserDataSyncTrigger };
